from werkzeug.datastructures import MultiDict

from pytest_httpserver.httpserver import BooleanQueryMatcher
from pytest_httpserver.httpserver import MappingQueryMatcher
from pytest_httpserver.httpserver import StringQueryMatcher


def assert_match(qm, query_string):
    values = qm.get_comparing_values(query_string)
    assert values[0] == values[1]


def assert_not_match(qm, query_string):
    values = qm.get_comparing_values(query_string)
    assert values[0] != values[1]


def test_qm_string():
    qm = StringQueryMatcher("k1=v1&k2=v2")
    assert_match(qm, b"k1=v1&k2=v2")
    assert_not_match(qm, b"k2=v2&k1=v1")


def test_qm_bytes():
    qm = StringQueryMatcher(b"k1=v1&k2=v2")
    assert_match(qm, b"k1=v1&k2=v2")
    assert_not_match(qm, b"k2=v2&k1=v1")


def test_qm_boolean():
    qm = BooleanQueryMatcher(True)
    assert_match(qm, b"k1=v1")


def test_qm_mapping_string():
    qm = MappingQueryMatcher({"k1": "v1"})
    assert_match(qm, b"k1=v1")


def test_qm_mapping_unordered():
    qm = MappingQueryMatcher({"k1": "v1", "k2": "v2"})
    assert_match(qm, b"k1=v1&k2=v2")
    assert_match(qm, b"k2=v2&k1=v1")


def test_qm_mapping_first_value():
    qm = MappingQueryMatcher({"k1": "v1"})
    assert_match(qm, b"k1=v1&k1=v2")

    qm = MappingQueryMatcher({"k1": "v2"})
    assert_match(qm, b"k1=v2&k1=v1")


def test_qm_mapping_multiple_values():
    md = MultiDict([("k1", "v1"), ("k1", "v2")])
    qm = MappingQueryMatcher(md)
    assert_match(qm, b"k1=v1&k1=v2")



import threading
from queue import Queue

import pytest
import requests

from pytest_httpserver import BlockingHTTPServer

# override httpserver fixture


@pytest.fixture
def httpserver():
    server = BlockingHTTPServer(timeout=1)
    server.start()

    yield server

    server.clear()
    if server.is_running():
        server.stop()

    # this is to check if the client has made any request where no
    # `assert_request` was called on it from the test

    server.check_assertions()
    server.clear()


def test_simplified(httpserver: BlockingHTTPServer):
    def client(response_queue: Queue):
        response = requests.get(httpserver.url_for("/foobar"), timeout=10)
        response_queue.put(response)

    # start the client, server is not yet configured
    # it will block until we add a request handler to the server
    # (see the timeout parameter of the http server)
    response_queue: Queue[requests.models.Response] = Queue(maxsize=1)
    thread = threading.Thread(target=client, args=(response_queue,))
    thread.start()

    try:
        # check that the request is for /foobar and it is a GET method
        # if this does not match, it will raise AssertionError and test will fail
        client_connection = httpserver.assert_request(uri="/foobar", method="GET")

        # with the received client_connection, we now need to send back the response
        # this makes the request.get() call in client() to return
        client_connection.respond_with_json({"foo": "bar"})

    finally:
        # wait for the client thread to complete
        thread.join(timeout=1)
        assert not thread.is_alive()  # check if join() has not timed out

    # check the response the client received
    response = response_queue.get(timeout=1)
    assert response.status_code == 200
    assert response.json() == {"foo": "bar"}



import json

import pytest
import requests

from pytest_httpserver import HTTPServer


def test_json_matcher(httpserver: HTTPServer):
    httpserver.expect_request("/foo", json={"foo": "bar"}).respond_with_data("Hello world!")
    assert requests.get(httpserver.url_for("/foo")).status_code == 500
    resp = requests.get(httpserver.url_for("/foo"), json={"foo": "bar"})
    assert resp.status_code == 200
    assert resp.text == "Hello world!"
    assert requests.get(httpserver.url_for("/foo"), json={"foo": "bar", "foo2": "bar2"}).status_code == 500


def test_json_matcher_with_none(httpserver: HTTPServer):
    httpserver.expect_request("/foo", json=None).respond_with_data("Hello world!")
    resp = requests.get(httpserver.url_for("/foo"), data=json.dumps(None), headers={"content-type": "application/json"})
    assert resp.status_code == 200
    assert resp.text == "Hello world!"


def test_json_matcher_without_content_type(httpserver: HTTPServer):
    httpserver.expect_request("/foo", json={"foo": "bar"}).respond_with_data("Hello world!")
    assert requests.get(httpserver.url_for("/foo"), json={"foo": "bar"}).status_code == 200
    assert requests.get(httpserver.url_for("/foo"), data=json.dumps({"foo": "bar"})).status_code == 200


def test_json_matcher_with_invalid_json(httpserver: HTTPServer):
    httpserver.expect_request("/foo", json={"foo": "bar"}).respond_with_data("Hello world!")
    assert requests.get(httpserver.url_for("/foo"), data="invalid-json").status_code == 500
    assert requests.get(httpserver.url_for("/foo"), data='{"invalid": "json"').status_code == 500
    assert requests.get(httpserver.url_for("/foo"), data=b"non-text\x1f\x8b").status_code == 500


def test_data_and_json_mutually_exclusive(httpserver: HTTPServer):
    with pytest.raises(ValueError):
        httpserver.expect_request("/foo", json={}, data="foo")
